/***************************************************************************
 *   Copyright (C) 2004 by Nicolas Boichat                                 *
 *   nicolas@boichat.ch                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include <stdio.h>
#include <unistd.h>
#include <sys/time.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <math.h>

#include "notebook.h"

#define GTK_FILL_EXPAND (GtkAttachOptions)(GTK_FILL|GTK_EXPAND)

struct param {
	int index;
	char* text;
	GtkWidget* widget;
	gulong handler;
};

struct param P(int index, char* text) {
	struct param p;
	p.index = index;
	p.text = text;
	p.widget = NULL;
	p.handler = 0;
	return p;
}

struct param params[][24] = {
	{P(0x10, "Brightness"), P(0x12, "Contrast"), P(-1, "General")},
	{P(0x16, "Red"), P(0x18, "Green"), P(0x1A, "Blue"), P(-1, "Color")},
	{P(0x22, "Width"), P(0x32, "Height"), P(-2, ""),
	P(0x20, "Horizontal\nposition"), P(0x30, "Vertical\nposition"), P(-1, "Size/position")},
	{P(0x24, "Pincushion\namplitude"), P(0x42, "Pincushion\nphase"), P(-2, ""),
	P(0x40, "Keystone\nbalance"), P(0x26, "Pincushion\nbalance"), P(-2, ""),
	P(0x46, "Top pincushion\namplitude"), P(0x48, "Top pincushion\nbalance"), P(-2, ""),
	P(0xE6, "Center pincushion\namplitude"), P(-2, ""),
	P(0x4A, "Bottom pincushion\namplitude"), P(0x4C, "Bottom pincushion\nbalance"), P(-2, ""),
	P(-1, "Geometry")},
	{P(0x3A, "Vertical linearity"), P(0x3C, "Vertical linearity\nbalance"), P(-2, ""),
	P(0x28, "Horizontal static\nconvergence"), P(0x38, "Vertical static\nconvergence"), P(-2, ""),
	P(0x56, "Moire cancel"), P(-2, ""),
	P(0xE8, "Top left\nscreen purity"), P(0xE9, "Top right\nscreen purity"), P(-2, ""),
	P(0xEA, "Bottom left\nscreen purity"), P(0xEB, "Bottom right\nscreen purity"), P(-2, ""),
	P(-1, "Geometry 2 & purity")},
	{P(-1, "")},
	{P(0x00, "Degauss (to be implemented value 0x01 degauss)")},
};

class buslist { /* Class, so its destructor is always called and the file handle are released */
	public:
		buslist() {
			probed = FALSE;
		}
		
		~buslist() {
			for (int i = 0; (i < 16) && (list[i] != NULL); i++) {
				delete list[i];
			}
		}
		
		ddc_control** getBusList() {
			if (probed)
				return (ddc_control**)&list;
			int bus;
			int busmax = 32;
			int i = 0;
						
			for (bus = 0; bus < busmax; bus++) {
				printf("Probing bus %d...", bus);
				list[i] = new ddc_control(bus);
				if (list[i]->init()) {
					printf("OK.\n");
//					printinfo(list[i]);
					list[++i] = NULL;
				}
				else {
					if (list[i]->getError() == DDC_PERMISSION_DENIED) {
						printf("Error, permission denied.\n");
						busmax = 32;
					}
					else if (list[i]->getError() == DDC_NO_SUCH_FILE_ERROR) {
						printf("None.\n");
						if (busmax-bus > 3) { /* If a file doesn't exists, read only three more */
							busmax = bus + 3;
						}
					}
					else if (list[i]->getError() == DDC_UNKNOWN_OPEN_ERROR) {
						printf("Error while opening.\n");
						busmax = 32;
					}
					else {
						printf("Probably not a DDC bus (error : %d).\n", list[i]->getError());
						busmax = 32;
					}
					delete list[i];
					list[i] = NULL;
				}
			}
			probed = TRUE;
			return (ddc_control**)&list;
		}
		
	private:
		int probed;
		ddc_control* list[16];
};

buslist bl;

ddc_control* currentdc = NULL;
GtkWidget* valuelabel;
GtkWidget* valuerange;
GtkWidget* restorevalue;
GtkWidget* currentbutton = NULL;
int currentIndex = -1;
unsigned char currentDefault = 1;
unsigned char currentMaximum = 1;

ddc_control** getBusList() {
	return bl.getBusList();
}

static void range_callback(GtkWidget *widget, gpointer data) {
	double val = gtk_range_get_value(GTK_RANGE(valuerange))/100.0;
	
	//printf("Would change %#x to %#x (%f)\n", currentIndex, (unsigned char)round(val*(double)currentMaximum), val);
	
	if ((currentdc != NULL) && (currentIndex != -1)) {
		unsigned char nval = (unsigned char)round(val*(double)currentMaximum);
		currentdc->setValue(currentIndex, nval);
		
		gtk_range_set_value(GTK_RANGE(valuerange), (double)100.0*nval/(double)currentMaximum);
	}
}

static void restore_callback(GtkWidget *widget, gpointer data) {	
	if ((currentdc != NULL) && (currentIndex != -1)) {
		currentdc->setValue(currentIndex, currentDefault);
		
		gtk_range_set_value(GTK_RANGE(valuerange), (double)100.0*currentDefault/(double)currentMaximum);
	}
}

static void buttons_callback(GtkWidget *widget, gpointer data) {
	if (currentbutton != NULL) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (currentbutton), 0);
	}
	
	struct param* p = (struct param*)data;
	
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) {
		currentbutton = widget;
		currentIndex = p->index;
		gtk_label_set_text(GTK_LABEL(valuelabel), p->text);
		if ((currentdc != NULL) && (currentdc->getValue(currentIndex, &currentDefault, &currentMaximum))) {
			/*printf("def: %d, max: %d\n", currentDefault, currentMaximum);
			printf("incs: %f, val: %f\n", 1.0/(double)currentMaximum, (double)currentDefault/(double)currentMaximum);*/
			gtk_range_set_increments(GTK_RANGE(valuerange), 100.0/(double)currentMaximum, 10.0*100.0/(double)currentMaximum);
			gtk_range_set_value(GTK_RANGE(valuerange), (double)100.0*currentDefault/(double)currentMaximum);
		}
		else {
			fprintf(stderr, "Error while getting value\n");
		}
		gtk_widget_show(restorevalue);
		gtk_widget_show(valuerange);
	} 
	else {
		currentbutton = NULL;
		currentIndex = -1;
		gtk_label_set_text(GTK_LABEL(valuelabel), "Please click on a parameter to the left to change it.");
		gtk_widget_hide(restorevalue);
		gtk_widget_hide(valuerange);
	}
}

int createPage(GtkWidget* notebook, int index) {
	int i;
	int x, y;
	int width = 0;
	GtkWidget* table = gtk_table_new(1, 1, TRUE);
	GtkWidget* button;
	GtkWidget* label;
	
	for (i = 0, x = 0, y = 0; params[index][i].index != -1; i++) {
		if (params[index][i].index == -2) {
			y++;
			if (x > width)
				width = x;
			x = 0;
			continue;
		}
		button = gtk_toggle_button_new();
		label = gtk_label_new(params[index][i].text);
		gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_CENTER);
		gtk_widget_show (label);
		gtk_container_add (GTK_CONTAINER (button), label);
		
		gtk_widget_show (button);
		gtk_table_attach(GTK_TABLE(table), button, x, x+1, y, y+1, GTK_FILL_EXPAND, GTK_EXPAND, 5, 5);
		if (params[index][i].widget != NULL) {
			g_signal_handler_disconnect(params[index][i].widget, params[index][i].handler);
		}
		params[index][i].widget = button;
		params[index][i].handler = 
			g_signal_connect_after(G_OBJECT(button), "toggled", 
					G_CALLBACK(buttons_callback), (gpointer)&params[index][i]);
		x++;
	}
	
	if (x > width)
		width = x;
	
	gtk_table_resize(GTK_TABLE(table), y+1, width);
	
	label = gtk_label_new (params[index][i].text);
	gtk_widget_show (label);
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), table, label);
	gtk_widget_show (table);
	
	return 1;
}

GtkWidget* createNotebook(ddc_control* dc) {
	currentbutton = NULL;
	currentIndex = -1;
	currentdc = dc;
	
	GtkWidget* notebook = gtk_notebook_new();
	
	for (int i = 0; params[i][0].index != -1; i++) {
		createPage(notebook, i);
	}
	
	GtkWidget* right = gtk_table_new(3, 1, TRUE);
	valuelabel = gtk_label_new ("Please click on a parameter to the left to change it.");
	gtk_table_attach(GTK_TABLE(right), valuelabel, 0, 1, 0, 1, GTK_FILL_EXPAND, GTK_EXPAND, 5, 5);
	valuerange = gtk_hscale_new_with_range(0.0, 100.0, 1.0 );
	gtk_widget_show(valuelabel);
	//gtk_widget_show(valuerange);
	gtk_scale_set_digits(GTK_SCALE(valuerange), 1);
	gtk_table_attach(GTK_TABLE(right), valuerange, 0, 1, 1, 2, GTK_FILL_EXPAND, GTK_EXPAND, 5, 5);
	g_signal_connect_after(G_OBJECT(valuerange), "value-changed", G_CALLBACK(range_callback), NULL);
	
	restorevalue = gtk_button_new_with_label("Restore default value");
	gtk_table_attach(GTK_TABLE(right), restorevalue, 0, 1, 2, 3, GTK_EXPAND, GTK_EXPAND, 5, 5);
	g_signal_connect_after(G_OBJECT(restorevalue), "clicked", G_CALLBACK(restore_callback), NULL);
	
	GtkWidget* table = gtk_table_new(1, 2, TRUE);
	gtk_widget_show (notebook);
	gtk_table_attach(GTK_TABLE(table), notebook, 0, 1, 0, 1, GTK_FILL_EXPAND, GTK_FILL_EXPAND, 3, 0);
	gtk_widget_show (right);
	gtk_table_attach(GTK_TABLE(table), right, 1, 2, 0, 1, GTK_FILL_EXPAND, GTK_EXPAND, 3, 0);
	gtk_widget_show (table);
	
	return table;
}

