/***************************************************************************
 *   Copyright (C) 2004 by Nicolas Boichat                                 *
 *   nicolas@boichat.ch                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 *   DDC is a Trademark of VESA (Video Electronics Standard Association).  *
 *                                                                         *
 ***************************************************************************/

#ifndef DDCCONTROL_H
#define DDCCONTROL_H

#define DDC_NO_ERROR 0

#define DDC_NOT_INITED 1

#define DDC_PERMISSION_DENIED 3
#define DDC_UNKNOWN_OPEN_ERROR 4
#define DDC_NO_SUCH_FILE_ERROR 5

#define DDC_INVALID_EDID 6

#define DDC_READ_ERROR 10
#define DDC_WRITE_ERROR 11
#define DDC_READCHKSUM_ERROR 12

#define DDC_BAD_ANSWER 15

class ddc_control {
public:
	//static ddc_control* probenext(ddc_control* dc = NULL);

	ddc_control(int bus);
	~ddc_control();

	int  init();
	int  getValue(unsigned char index, unsigned char* value, unsigned char* max);
	int  setValue(unsigned char index, unsigned char value);

	int  getError();

	int getBus();

	const char* getManufacturer();
	const int getModel();
	const long getSerial();
	const int getYear();
	const int getWeek();
	
	/* Return a printable name */
	const char* getName();

private:
	int  get_EDID();

	void addchecksum(unsigned char* buffer);
	int  checksum(unsigned char* buffer, unsigned char subaddr);

/* EDID functions */
	int edid_checksum(unsigned char *edid);
	int edid_check_header(unsigned char *edid);

/* Private members */
	int fd;
	int error;
	int bus; //I2C bus

	char manufacturer[4];
	int model;
	long serial;
	int year;
	int week;
	
	char name[64];
};

#endif
